﻿using System;
using System.IO;
using Google.ProtocolBuffers;
using System.Xml;
using Logging.Properties;

namespace Logging
{
    public class GenerateLogs
    {
        const int Files = 100;
        const int EntriesPerFile = 100000;

        private static readonly string[] Users = { "Jon", "Holly", "Tom", "Robin", "William", "Matt", "Nick", "Max", "Zack" };
        private static readonly Severity[] AllSeverities = (Severity[]) Enum.GetValues(typeof(Severity));
        private static readonly string[] Messages =
        {
            "Logged in",
            "Completed transaction",
            "Logged out",
            "Performed search",
            "Edited preferences",
            "Canceled request"
        };                      

        public static void Main()
        {
            Random rng = new Random();
            for (int i = 0; i < Files; i++)
            {
                string filename = Guid.NewGuid() + ".log";
                using (Stream stream = File.Create(Path.Combine(Settings.Default.LogDirectory, filename)))
                {
                    for (int j=0; j < EntriesPerFile; j++)
                    {
                        MessageStreamWriter<LogEntry> writer = new MessageStreamWriter<LogEntry>(stream);
                        LogEntry.Builder builder = new LogEntry.Builder();

                        builder.User = rng.Next(Users);
                        builder.Severity = rng.Next(AllSeverities);
                        builder.Date = new DateTime(2005 + rng.Next(3), rng.Next(12) + 1, rng.Next(28) + 1,
                            rng.Next(24), rng.Next(60), rng.Next(60));
                        builder.Message = rng.Next(Messages);
                        if (builder.Severity == Severity.ERROR || builder.Severity == Severity.FATAL)
                        {
                            builder.Exception = LogException.FromException(GenerateException(rng));
                        }

                        writer.Write(builder.Build());
                        writer.Flush();

                    }
                }
            }
        }

        private static Exception GenerateException(Random rng)
        {
            // Throw and catch the exception so we've got a stack trace
            try
            {
                Exception innerException = null;
                if (rng.Next(4) == 0)
                {
                    innerException = GenerateException(rng);
                }
                switch (rng.Next(5))
                {
                    case 0:
                        throw new ArgumentException(GenerateExceptionMessage(rng), innerException);
                    case 1:
                        throw new IOException(GenerateExceptionMessage(rng), innerException);
                    case 2:
                        throw new InvalidOperationException(GenerateExceptionMessage(rng), innerException);
                    case 3:
                        throw new XmlException(GenerateExceptionMessage(rng), innerException);
                    case 4:
                        throw new NullReferenceException();
                }
            }
            catch (Exception e)
            {                
                return e;
            }
            throw new InvalidOperationException();
        }

        private static readonly string[] MessageSubjects = { "The frobulator", "Data", "User", "Programmer", "Julius Caesar" };
        private static readonly string[] MessageVerbs = { " should ", " did not ", " can't ", " must not on any account " };
        private static readonly string[] MessageTargets = { "explode.", "provide a parameter.", "bind to Kermit.", "read the file."};

        private static string GenerateExceptionMessage(Random rng)
        {
            return rng.Next(MessageSubjects) + rng.Next(MessageVerbs) + rng.Next(MessageTargets);
        }
    }
}
